@@ -41,6 +41,11 @@ from mostlyai.engine._common import (
     ProgressCallback,
     ProgressCallbackWrapper,
 )
+from mostlyai.engine._encoding_types.language.datetime import (
+    analyze_reduce_language_datetime,
+    analyze_language_datetime,
+)
+from mostlyai.engine._encoding_types.language.numeric import analyze_language_numeric, analyze_reduce_language_numeric
 from mostlyai.engine._encoding_types.tabular.categorical import (
     analyze_categorical,
     analyze_reduce_categorical,
@@ -66,6 +71,10 @@ from mostlyai.engine._encoding_types.language.text import (
     analyze_text,
     analyze_reduce_text,
 )
+from mostlyai.engine._encoding_types.language.categorical import (
+    analyze_language_categorical,
+    analyze_reduce_language_categorical,
+)
 from mostlyai.engine.domain import ModelEncodingType
 
 from mostlyai.engine._workspace import (
@@ -84,6 +93,9 @@ _VALUE_PROTECTION_ENCODING_TYPES = (
     ModelEncodingType.tabular_numeric_binned,
     ModelEncodingType.tabular_datetime,
     ModelEncodingType.tabular_datetime_relative,
+    ModelEncodingType.language_categorical,
+    ModelEncodingType.language_numeric,
+    ModelEncodingType.language_datetime,
 )
 
 
@@ -313,22 +325,7 @@ def _analyze_reduce(
         column: column_stats.get("encoding_type") for column, column_stats in stats_list[0]["columns"].items()
     }
 
-    # build mapping of original column name to ARGN table and column identifiers
-    def get_table(qualified_column_name: str) -> str:
-        # column names are assumed to be <table>::<column>
-        return qualified_column_name.split(TABLE_COLUMN_INFIX)[0]
-
-    def get_unique_tables(qualified_column_names: Iterable[str]) -> list[str]:
-        duplicated_tables = [get_table(c) for c in qualified_column_names]
-        return list(dict.fromkeys(duplicated_tables))
-
-    unique_tables = get_unique_tables(encoding_types.keys())
-    argn_identifiers: dict[str, tuple[str, str]] = {
-        c: (f"t{unique_tables.index(get_table(qualified_column_name=c))}", f"c{idx}")
-        for idx, c in enumerate(encoding_types.keys())
-    }
-
-    for i, column in enumerate(encoding_types.keys()):
+    for column in encoding_types:
         encoding_type = encoding_types[column]
         column_stats_list = [item["columns"][column] for item in stats_list]
         column_stats_list = [
@@ -379,6 +376,21 @@ def _analyze_reduce(
             )
         elif encoding_type == ModelEncodingType.language_text:
             stats_col = analyze_reduce_text(stats_list=column_stats_list)
+        elif encoding_type == ModelEncodingType.language_categorical:
+            stats_col = analyze_reduce_text(stats_list=column_stats_list) | analyze_reduce_language_categorical(
+                stats_list=column_stats_list,
+                value_protection=value_protection,
+            )
+        elif encoding_type == ModelEncodingType.language_numeric:
+            stats_col = analyze_reduce_text(stats_list=column_stats_list) | analyze_reduce_language_numeric(
+                stats_list=column_stats_list,
+                value_protection=value_protection,
+            )
+        elif encoding_type == ModelEncodingType.language_datetime:
+            stats_col = analyze_reduce_text(stats_list=column_stats_list) | analyze_reduce_language_datetime(
+                stats_list=column_stats_list,
+                value_protection=value_protection,
+            )
         else:
             raise RuntimeError(f"unknown encoding type {encoding_type}")
 
@@ -388,29 +400,53 @@ def _analyze_reduce(
         if encoding_type in _VALUE_PROTECTION_ENCODING_TYPES:
             stats_col = {"value_protection": value_protection} | stats_col
 
-        # select model pipeline to process given column
-        def get_argn_processor(mode, is_flat) -> str:
-            if mode == "tgt":
-                return TGT
-            else:  # mode == "ctx"
-                return CTXFLT if is_flat else CTXSEQ
-
-        is_flat = "seq_len" not in column_stats_list[0]
-        stats_col[ARGN_PROCESSOR] = get_argn_processor(mode, is_flat)
-        (
-            stats_col[ARGN_TABLE],
-            stats_col[ARGN_COLUMN],
-        ) = argn_identifiers[column]
-
-        if not is_flat:
+        is_flat_column = "seq_len" not in column_stats_list[0]
+        if not is_flat_column:
             stats_col["seq_len"] = _analyze_reduce_seq_len([column_stats_list[0]["seq_len"]])
 
-        if encoding_type == ModelEncodingType.language_text:
-            _LOG.info(
-                f"analyzed column `{column}`: {stats_col['encoding_type']} nchar_max={stats_col['nchar_max']} nchar_avg={stats_col['nchar_avg']}"
+        is_language_column = encoding_type in (
+            ModelEncodingType.language_text,
+            ModelEncodingType.language_categorical,
+            ModelEncodingType.language_numeric,
+            ModelEncodingType.language_datetime,
+        )
+
+        if not is_language_column:
+            # build mapping of original column name to ARGN table and column identifiers
+            def get_table(qualified_column_name: str) -> str:
+                # column names are assumed to be <table>::<column>
+                return qualified_column_name.split(TABLE_COLUMN_INFIX)[0]
+
+            def get_unique_tables(qualified_column_names: Iterable[str]) -> list[str]:
+                duplicated_tables = [get_table(c) for c in qualified_column_names]
+                return list(dict.fromkeys(duplicated_tables))
+
+            unique_tables = get_unique_tables(encoding_types.keys())
+            argn_identifiers: dict[str, tuple[str, str]] = {
+                c: (f"t{unique_tables.index(get_table(qualified_column_name=c))}", f"c{idx}")
+                for idx, c in enumerate(encoding_types.keys())
+            }
+
+            def get_argn_processor(mode, is_flat) -> str:
+                if mode == "tgt":
+                    return TGT
+                else:  # mode == "ctx"
+                    return CTXFLT if is_flat else CTXSEQ
+
+            stats_col[ARGN_PROCESSOR] = get_argn_processor(mode, is_flat="seq_len" not in column_stats_list[0])
+            (
+                stats_col[ARGN_TABLE],
+                stats_col[ARGN_COLUMN],
+            ) = argn_identifiers[column]
+
+        _LOG.info(
+            f"analyzed column `{column}`: {stats_col['encoding_type']} "
+            + (
+                f"nchar_max={stats_col['nchar_max']} nchar_avg={stats_col['nchar_avg']}"
+                if is_language_column
+                else f"{stats_col['cardinalities']}"
             )
-        else:
-            _LOG.info(f"analyzed column `{column}`: {stats_col['encoding_type']} {stats_col['cardinalities']}")
+        )
         stats["columns"][column] = stats_col
 
     if mode == "ctx":
@@ -513,6 +549,18 @@ def _analyze_flat_col(
         stats = analyze_latlong(values, root_keys, context_keys)
     elif encoding_type == ModelEncodingType.language_text:
         stats = analyze_text(values, root_keys, context_keys)
+    elif encoding_type == ModelEncodingType.language_categorical:
+        stats = analyze_text(values, root_keys, context_keys) | analyze_language_categorical(
+            values, root_keys, context_keys
+        )
+    elif encoding_type == ModelEncodingType.language_numeric:
+        stats = analyze_text(values, root_keys, context_keys) | analyze_language_numeric(
+            values, root_keys, context_keys
+        )
+    elif encoding_type == ModelEncodingType.language_datetime:
+        stats = analyze_text(values, root_keys, context_keys) | analyze_language_datetime(
+            values, root_keys, context_keys
+        )
     else:
         raise RuntimeError(f"unknown encoding type: `{encoding_type}` for `{values.name}`")
     return stats
